"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Server = void 0;
const apollo_server_express_1 = require("apollo-server-express");
const body_parser_1 = require("body-parser");
const express_1 = __importDefault(require("express"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const graphql_depth_limit_1 = __importDefault(require("graphql-depth-limit"));
const merge_1 = require("@graphql-tools/merge");
const util_1 = require("./util");
const apollo_server_plugins_1 = require("./apollo_server_plugins");
const errors_1 = require("./errors");
const express_middleware_1 = require("./express_middleware");
const ts_log_1 = require("ts-log");
const dynamic_1 = require("set-interval-async/dynamic");
const set_interval_async_1 = require("set-interval-async");
class Server {
    constructor(schemas, config, logger = ts_log_1.dummyLogger) {
        this.config = config;
        this.logger = logger;
        this.app = express_1.default();
        this.state = null;
        this.schemas = schemas;
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.state !== null)
                return;
            this.state = 'initializing';
            this.logger.info({ module: 'Server' }, 'Initializing');
            let allowList;
            const plugins = [];
            const validationRules = [];
            const schema = merge_1.mergeSchemas({
                schemas: this.schemas
            });
            if (this.config.allowListPath) {
                if (this.config.allowIntrospection === true) {
                    throw new errors_1.IntrospectionNotPermitted('allowListPath');
                }
                if (this.config.maxQueryComplexity) {
                    throw new errors_1.QueryComplexityLimitationNotPermitted('allowListPath');
                }
                try {
                    const file = yield fs_extra_1.default.readFile(this.config.allowListPath, 'utf8');
                    allowList = JSON.parse(file);
                    this.app.use('/', body_parser_1.json(), express_middleware_1.allowListMiddleware(allowList));
                    this.logger.info({ module: 'Server' }, 'The server will only allow only operations from the provided list');
                }
                catch (error) {
                    this.logger.error({ module: 'Server' }, `Cannot read or parse allow-list JSON file at ${this.config.allowListPath}`);
                    throw error;
                }
            }
            if (this.config.prometheusMetrics) {
                if (this.config.tracing === false) {
                    throw new errors_1.TracingRequired('Prometheus');
                }
                plugins.push(apollo_server_plugins_1.prometheusMetricsPlugin(this.app));
                this.logger.info({ module: 'Server' }, 'Prometheus metrics will be served at /metrics');
            }
            if (this.config.queryDepthLimit) {
                validationRules.push(graphql_depth_limit_1.default(this.config.queryDepthLimit));
            }
            if (this.config.maxQueryComplexity) {
                plugins.push(apollo_server_plugins_1.queryComplexityPlugin(schema, this.logger, this.config.maxQueryComplexity));
            }
            this.apolloServer = new apollo_server_express_1.ApolloServer({
                cacheControl: this.config.cacheEnabled ? { defaultMaxAge: 20 } : undefined,
                introspection: this.config.allowIntrospection,
                playground: this.config.allowIntrospection,
                plugins,
                validationRules,
                schema,
                tracing: this.config.tracing
            });
            this.apolloServer.applyMiddleware({
                app: this.app,
                cors: this.config.allowedOrigins ? {
                    origin: this.config.allowedOrigins
                } : undefined,
                path: '/'
            });
            this.state = 'initialized';
        });
    }
    start() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.state !== 'initialized')
                return;
            this.httpServer = yield util_1.listenPromise(this.app, this.config.apiPort, this.config.listenAddress);
            this.state = 'running';
            this.logger.info({ module: 'Server' }, `GraphQL HTTP server at http://${this.config.listenAddress}:` +
                `${this.config.apiPort}${this.apolloServer.graphqlPath} started`);
            this.logger.debug({ module: 'Server' }, 'Checking DB status');
            this.syncProgress = dynamic_1.setIntervalAsync(() => __awaiter(this, void 0, void 0, function* () {
                const result = yield this.apolloServer.executeOperation({
                    query: `query getSyncStatus {
              cardanoDbMeta {
                  initialized
                  syncPercentage
              }
          }`
                });
                if (result.errors !== undefined) {
                    this.logger.debug({ module: 'Server' }, JSON.stringify(result.errors));
                    return;
                }
                if (result.data.cardanoDbMeta.initialized) {
                    this.logger.info({ module: 'Server' }, 'DB ready');
                    set_interval_async_1.clearIntervalAsync(this.syncProgress);
                }
                else {
                    this.logger.info({ module: 'Server' }, `DB sync progress: ${result.data.cardanoDbMeta.syncPercentage} %`);
                }
            }), 5000);
        });
    }
    shutdown() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.state !== 'running')
                return;
            yield set_interval_async_1.clearIntervalAsync(this.syncProgress);
            this.httpServer.close();
            this.logger.info({ module: 'Server' }, `GraphQL HTTP server at http://${this.config.listenAddress}:` +
                `${this.config.apiPort}${this.apolloServer.graphqlPath} shutting down`);
            this.state = 'initialized';
        });
    }
}
exports.Server = Server;
//# sourceMappingURL=Server.js.map